import json
import inspect
import os
import re
import ix
import logging
import sys

supported_3dfileformats = [".obj" , ".abc"]
material_presets_3dasset = ["disneyprincipled", "standard"]
nonlinear_maps = ["albedo", "diffuse", "translucency", "specular"]
maps_not_allowed = []

# Logger for logging livelink activity
def setup_custom_logger(name):
    try :
        log_file_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        print log_file_path
        if not os.access(log_file_path, os.W_OK):
            log_file_path = name+".log"
        else :            
            log_file_path = os.path.join(log_file_path , str(name+".log"))
        formatter = logging.Formatter(fmt='%(asctime)s %(levelname)-8s %(message)s',
                                  datefmt='%Y-%m-%d %H:%M:%S')
        handler = logging.FileHandler(log_file_path, mode='a+')
        handler.setFormatter(formatter)
        screen_handler = logging.StreamHandler(stream=sys.stdout)
        screen_handler.setFormatter(formatter)
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)
        logger.addHandler(screen_handler)

        return logger

    except Exception as e:
        return None

logger = None
logger = setup_custom_logger("ms_clarisse")

# Write to the log file in the LiveLink folder, logger created with setup_custom_logger() function
def write_to_log(log_type, log_message):
    if logger == None:
        return
    if log_type== "info":
        logger.info(log_message)
    elif log_type == "error":
        logger.error(log_message)
    elif log_type == "warning":
        logger.warning(log_message)


# Main entry function that is called through the Command Port from the MS_ClLiveLink file/executable, json from Bridge is passed as unicode
def ms_import_asset(import_data):
    # global logger   

    json_array = json.loads(import_data)    
    parse_import_json(json_array)


    # logging.shutdown()

# This function parses the json data from bridge and calls relevant functions for each asset
def parse_import_json(import_data_json):
    for asset_data in import_data_json:
        write_to_log("info" , "\n")
        write_to_log("info" , "######################################################################################")
        write_to_log("info" , "Incoming asset json : %s" %asset_data)
        
        write_to_log("info", "Importing asset : id=%s , name=%s" % (str(asset_data["id"]), str(asset_data["name"])))
        parse_asset_data(asset_data)


# Prases json for each asset, parameter passed contains the single asset json data
def parse_asset_data(asset_data):
    
    try :    
        asset_type = asset_data["type"]
        
        if asset_type == "3d":            
            import_3d(asset_data)
        elif asset_type == "3dplant":
            import_3d(asset_data)
        elif asset_type == "atlas":
            import_non3d(asset_data)
        elif asset_type == "surface":
            import_non3d(asset_data)
        elif asset_type == "brush":
            import_non3d(asset_data)
    except Exception as e :
        write_to_log("error", "Error importing asset : %s" %str(e))
        

# Imports all meshes in the meshList key of json from Bridge
def import_3dasset(asset_data, asset_context):
    try :
        polyfileobjects = []    
        for mesh in asset_data["meshList"]:            
            mesh_filepath = mesh["path"]            
            filepath, fileextension = os.path.splitext(os.path.basename(mesh_filepath))
            polyfile_name = get_name_override(mesh["nameOverride"], mesh["name"])            

            # if str(mesh["name"]) == str(mesh["nameOverride"]):
            #     if str( asset_data["type"]) == "3d":
            #         polyfile_name =  normalize_name ( str(asset_data["name"]) +"_" + str(asset_data["id"]))

            if str(asset_data["type"]) == "3dplant" and str(mesh["nameOverride"]) == mesh["name"]:
                polyfile_name = normalize_name ( str(asset_data["name"]) +"_" + str(asset_data["id"]) + "_" + polyfile_name )

            if fileextension not in supported_3dfileformats :
                write_to_log("warning", "Mesh file format not supported for asset : %s " % mesh_filepath)
                # return None
                break

            polyfileobject = ix.cmds.CreateObject(str(polyfile_name), "GeometryPolyfile", "Global", asset_context)
            write_to_log( "info", "Mesh object created ")

            ix.cmds.SetValues([str(polyfileobject) + ".filename"], [str(mesh_filepath)])
            write_to_log( "info", "Mesh object file path set to : %s" % str(mesh_filepath))

            polyfileobject.attrs.scale_offset[0] = .01
            polyfileobject.attrs.scale_offset[1] = .01
            polyfileobject.attrs.scale_offset[2] = .01
            polyfileobjects.append(polyfileobject)
        return polyfileobjects

    except Exception as e :
        write_to_log("error", "Error importing asset : %s" %str(e))
        return []


# To get the asset name by combining its name and id to create a unique name. To distinguish between same name assets.
def get_asset_name(asset_name, asset_id):
    asset_name =  str(asset_name) + "_" + str(asset_id)
    asset_name = normalize_name (asset_name)
    return asset_name

# For all assets containing 3d data (meshes)
def import_3d (asset_data):
    asset_name = get_asset_name (asset_data["name"], asset_data["id"])
    asset_context = create_asset_context(asset_data["exportPath"], asset_name, str(asset_data["type"]))
    material_context = create_assetmtl_context(asset_context)

    asset_polymeshes = import_3dasset(asset_data, asset_context)



    
    # If any mesh was sucessfully downloaded
    if len(asset_polymeshes) > 0:
        if not None in asset_polymeshes:
            write_to_log( "info", "Creating material for the asset ")
            asset_material = create_asset_material(asset_data["components"], material_context, asset_name, asset_polymeshes)

            if asset_material is not None:
                for polyobject in asset_polymeshes:
                    geo = polyobject.get_module()
                    geo.assign_material(asset_material.get_module(), 0)
                # geo = asset_polymesh.get_module()
                # geo.assign_material(asset_material.get_module(), 0)
                write_to_log( "info", "Material applied to asset")

    else :
        write_to_log( "error", "Problem importing mesh, deleting asset context")
        ix.cmds.DeleteItems([asset_context])

def import_non3d (asset_data):
    asset_name = get_asset_name (asset_data["name"], asset_data["id"])
    asset_context = create_asset_context(asset_data["exportPath"], asset_name, str(asset_data["type"]))
    create_asset_material(asset_data["components"], asset_context, asset_name, asset_type=str(asset_data["type"]))



############ Material Setup ############
# Create the material, used for all types of materials
def create_asset_material(texture_data, asset_context, asset_name, asset_polymeshes=None,  material_type="disney_principled", asset_type="standard"):
    default_materials = ["disney_principled" , "standard"]
    asset_material = None
    texture_maps = import_texture_maps ( texture_data, asset_context)

    if asset_type == "brush":
        return

    color_node = create_color_node ( texture_maps, asset_context )
    write_to_log( "info", "Material color node created")

    normalbump_node = create_normalbump_node ( texture_maps, asset_context)
    write_to_log( "info", "Material normal/bump node created")

    if "displacement" in texture_maps:
        create_displacement(texture_maps["displacement"], asset_context, asset_name, asset_polymeshes)

    if "opacity" in texture_maps:
        create_opacity(texture_maps["opacity"], asset_context, asset_name, asset_polymeshes)

    asset_smaterial = create_standard_material(texture_maps, asset_name, asset_context,  color_node, normalbump_node, asset_polymeshes)

    asset_pdpmaterial =  create_pdpm(texture_maps, asset_name, asset_context,  color_node, normalbump_node, asset_polymeshes)

    if asset_pdpmaterial is not None:
        return asset_pdpmaterial

    elif asset_smaterial is not None:
        return  asset_smaterial

    write_to_log( "error", "Error creating material")
    return None

# Color node created for both Standard and Disney Principled, uses Albedo and Cavity maps
def create_color_node ( texture_maps, asset_context ):
    if "albedo" in texture_maps:
        albedo_map_file = texture_maps["albedo"]

        if "cavity" in texture_maps:
            cavity_map_file = texture_maps["cavity"]
            multiply_map = ix.cmds.CreateObject("multiply", "TextureMultiply", "Global", asset_context)
            ix.cmds.SetTexture([str(multiply_map) + ".input1"], albedo_map_file)
            ix.cmds.SetTexture([str(multiply_map) + ".input2"], cavity_map_file)
            return multiply_map

        else :
            return albedo_map_file
            #ix.cmds.SetTexture([str(base_material)+ ".base_color"], str(albedo_map_file))

    else :
        if "cavity" in texture_maps:
            return None

    write_to_log( "info", "Albedo texture and map not found")
    return None

# Create the Normal/Bump maps with subsequent native bump/normal nodes, either returns switch node if both maps are imported or any one of the nodes
def create_normalbump_node ( texture_maps, asset_context ):
    if "normal" in texture_maps:
        normal_map_file = texture_maps["normal"]
        normal_map_node = ix.cmds.CreateObject("normal_map", "TextureNormalMap", "Global", asset_context)
        ix.cmds.SetTexture([str(normal_map_node) +".input"], normal_map_file)

        if "bump" in texture_maps:
            bump_map_file = texture_maps["bump"]
            bump_map_node = ix.cmds.CreateObject("bump_map", "TextureBumpMap", "Global", asset_context)
            ix.cmds.SetTexture([str(bump_map_node) +".input"], bump_map_file)
            switch_node = ix.cmds.CreateObject("switch", "TextureSwitch", "Global", asset_context)
            ix.cmds.AddValues([str(switch_node)+".textures"], [normal_map_node])
            ix.cmds.AddValues([str(switch_node)+".textures"], [bump_map_node])
            return switch_node

        else :
            return normal_map_node

    else :
        if "bump" in texture_maps:
            bump_map_file = texture_maps["bump"]
            bump_map_node = ix.cmds.CreateObject("bump_map", "TextureBumpMap", "Global", asset_context)
            ix.cmds.SetTexture([str(bump_map_node) +".input"], bump_map_file)
            return bump_map_node

    return None

# Imports all the textures sent in the Bridge json, texture_data parameter is same "components" in json
def import_texture_maps (texture_data, asset_context):
    write_to_log( "info", "Importing textures and creating map files")
    constant_map = ix.cmds.CreateObject("constant_color", "TextureConstantColor", "Global", asset_context)
    texture_maps = {}
    for texture in texture_data:
        texture_type = str(texture["type"])
        texture_map = create_map(texture, asset_context)
        if texture_type not in nonlinear_maps:
            ix.cmds.SetValues([str(texture_map )+".use_raw_data"], ["1"])

        ix.cmds.SetTexture([str(texture_map )+".uv_scale"], constant_map)
        texture_maps[texture_type] = texture_map

    write_to_log( "info", "Successfully imported textures and created map files")
    return texture_maps

# texture_info parameter is same as an element in "components" in Bridge json
def create_map (texture_info, asset_context):
    map_name = get_name_override(texture_info["nameOverride"], texture_info["name"])    
    texture_path = str(texture_info["path"])
    write_to_log( "info", "Importing texture from path : %s" %texture_path)
    map_file = ix.cmds.CreateObject(map_name, "TextureMapFile", "Global", asset_context)
    ix.cmds.SetValues([str(map_file) + ".filename"], [texture_path])
    return map_file

# Creates a displacement node, creating the Clarisse displacement node and plugging the displacement map, also applies the displacement node to any 3d asset imported
def create_displacement(displacement_map, asset_context, asset_name, asset_polymesh=None):
    displacement_node = ix.cmds.CreateObject(str(asset_name+"_displacement"), "Displacement", "Global", asset_context)
    ix.cmds.SetTexture([str(displacement_node)+".front_value"], displacement_map)
    ix.cmds.SetValues([str(displacement_node) + ".front_value"], ["0.025"])

    if asset_polymesh == None:
        return

    for polymesh in asset_polymesh:
        if polymesh is not None:
            geo = polymesh.get_module()
            geo.assign_displacement(displacement_node.get_module(), 0)

# Sets up the opacity map and in case the surface is created for a mesh then opacity is applied to clip map on mesh
def create_opacity(opacity_map, asset_context, asset_name, asset_polymesh=None):
    ix.cmds.SetValues([str(opacity_map) +".single_channel_file_behavior"], ["1"])

    if asset_polymesh == None:
        return

    for polymesh in asset_polymesh:
        if polymesh is not None:
            geo = polymesh.get_module()
            geo.assign_clip_map(opacity_map.get_module(), 0)



############ Physical Disney Principled Material ############
def create_pdpm(texture_maps, asset_name, asset_context,  color_node, normalbump_node, asset_polymesh=None):
    write_to_log( "info", "Creating Physical Disney Principled Material")
    pdp_material = ix.cmds.CreateObject(asset_name+"_PDPMaterial", "MaterialPhysicalDisneyPrincipled", "Global", asset_context)
    if color_node is not None:
        ix.cmds.SetTexture([str(pdp_material)+ ".base_color"], color_node)

    if normalbump_node is not None:
        ix.cmds.SetTexture([str(pdp_material)+ ".normal_input"], normalbump_node)

    for texture_type, texture_map in texture_maps.items():
        if texture_type == "roughness":
            ix.cmds.SetTexture([str(pdp_material)+ ".roughness"], texture_map)
        elif texture_type == "specular":
            ix.cmds.SetTexture([str(pdp_material)+ ".specular"], texture_map)
        elif texture_type == "metalness":
            ix.cmds.SetTexture([str(pdp_material)+ ".metallic"], texture_map)

    write_to_log( "info", "Created Physical Disney Principled Material at context : %s" %asset_context)
    return pdp_material

def create_standard_material(texture_maps, asset_name, asset_context,  color_node, normalbump_node, asset_polymesh=None):
    write_to_log( "info", "Creating Standard Meterial")
    standard_material = ix.cmds.CreateObject(asset_name+"_SMaterial", "MaterialPhysicalStandard", "Global", asset_context)
    if color_node is not None:
        ix.cmds.SetTexture([str(standard_material)+ ".diffuse_front_color"], color_node)

    if normalbump_node is not None:
        ix.cmds.SetTexture([str(standard_material)+ ".normal_input"], normalbump_node)

    for texture_type, texture_map in texture_maps.items():
        if texture_type == "roughness":
            ix.cmds.SetTexture([str(standard_material)+ ".specular_1_roughness"], texture_map)

        elif texture_type == "specular":
            ix.cmds.SetTexture([str(standard_material)+ ".specular_1_color"], texture_map)

        elif texture_type == "translucency" :
            ix.cmds.SetTexture([str(standard_material)+ ".diffuse_back_color"], texture_map)
    write_to_log( "info", "Created Standard Material at context : %s" %asset_context)
    return standard_material


############ Create Megascans root and Asset context inside Clarisse ############
def create_asset_context(custom_context, asset_name, asset_type):
    type_context_name = {"3d": "3d_Assets", "3dplant" : "_3d_Plants", "atlas": "Atlasses", "surface":"Surfaces", "brush" : "Brushes"}
    custom_context = str(custom_context)
    base_context = create_base_context(custom_context)
    #type_context = ix.cmds.CreateContext(type_context_name[str(asset_type)], "Global", base_context)
    type_context = create_type_context(base_context, asset_type)

    asset_context = ix.cmds.CreateContext(str(asset_name), "Global", type_context)
    return asset_context

# In case a custom context is defined then create the custom context, currently the custom context functionality is not added
def create_base_context(custom_context):
    try:
        ctx = ix.get_item("project://scene/Megascans")

    except Exception as e:
        #check for custom context in json, create context accordingly
        msbase_context = ix.cmds.CreateContext("Megascans" , "Global", "project://scene")
        return msbase_context

    return ctx

# Material context created for any asset type that has 3d meshes included
def create_assetmtl_context( asset_context ):
    return ix.cmds.CreateContext("Material", "Global", asset_context)


# Base context for asset types
def create_type_context( base_context, asset_type):
    type_contexts = {"3d": "_3d_Assets", "3dplant" : "_3d_Plants", "atlas": "Atlasses", "surface":"Surfaces", "brush" : "Brushes"}
    type_context_name = type_contexts[asset_type]

    try:
        ctx = ix.get_item(str("project://scene/Megascans"+"/"+type_context_name))

    except Exception as e:
        print "No type context found"
        type_context = ix.cmds.CreateContext(type_context_name , "Global", base_context)
        return type_context

    return ctx


############ Utility Functions ############
# Returns the overriden name as set in custom naming conevntions in Bridge
def get_name_override( name_override, asset_name ):
    if str(name_override) is not "":
        asset_new_name = str(name_override).split(".")[0]

    else :
        asset_new_name = str(asset_name).split(".")[0]

    return normalize_name(asset_new_name).rstrip("_")

# Normalizes the name to remove any unsupported characters
def normalize_name (asset_name):
    return re.sub(r"[^a-zA-Z0-9]+" , "_", asset_name)



