import os
import sys
import platform
import logging
import fileinput
# from shutil import copyfile

base_path = os.path.dirname(os.path.realpath(__file__))
if len(sys.argv) > 1:
	base_path = sys.argv[1]
else :
	print "Destination argument not passed to the installer"
	sys.exit(1)

def setup_custom_logger(name,install_path):
    formatter = logging.Formatter(fmt='%(asctime)s %(levelname)-8s %(message)s',
                                  datefmt='%Y-%m-%d %H:%M:%S')
    handler = logging.FileHandler(install_path, mode='a+')
	# handler = logging.FileHandler(name+".log", mode='a+')
    handler.setFormatter(formatter)
    screen_handler = logging.StreamHandler(stream=sys.stdout)
    screen_handler.setFormatter(formatter)
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)
    logger.addHandler(screen_handler)
    return logger


install_log_path = os.path.join(base_path, "install.log")
logger = setup_custom_logger('install', install_log_path)

launcher_script_name = "MS_LiveLinkLauncher.py"
livelink_script_name = "MS_ClLiveLink.py"
livelink_exe_name = "MS_ClLiveLink.exe"
icon_file_name = "MS_Logo.png"

# logging.basicConfig(filename='clarisse.log',level=logging.DEBUG)



livelink_path = os.path.join(base_path, "MSLiveLink")

if not os.path.isdir(livelink_path):
	logger.error("Plugin directory doesnt exist at path : %s" %livelink_path)
	sys.exit(1)

launcher_path = os.path.join(livelink_path, launcher_script_name)
livelink_script_path = os.path.join(livelink_path, livelink_script_name)
livelink_exe_path = os.path.join(livelink_path, livelink_exe_name)

icon_path = os.path.join(livelink_path, icon_file_name)

supported_clarisse = ["4.0"]

if platform.system().lower() == "windows":
	launcher_path = launcher_path.replace("\\", "/")
	icon_path = icon_path.replace("\\", "/")

livelink_clarisse_cfg ='''
	category "Quixel Megascans" {
            shelf_item {
                title "MS_LiveLink"
                script_filename "%s"
                icon_filename "%s"
            }
        }'''%(launcher_path,icon_path)



def set_livelink_path():	
	try:
		launcher_script = open(launcher_path, "r+")
		content = launcher_script.read()
		launcher_script.seek(0, 0)
		launcher_string = 'LiveLink_Path = r"%s" \n' % livelink_script_path
		launcher_string = launcher_string + 'LiveLink_Exe_Path = r"%s" \n' % livelink_exe_path
		launcher_string += 'Package_Path = r"%s" \n' % base_path
		launcher_script.write(launcher_string + content)
		launcher_script.close()
		logger.info('Livelink paths added to script files')

	except Exception as e:
		logger.error("Error writing paths to script files")
		sys.exit(1)

remove_begin_string = 'category "Quixel Megascans" {'


def uninstall_livelink():
	try :
		logger.info("Removing existing versions of Quixel Megascans")
		clarisse_path = ""
		clarisse_config = "shelf.cfg"
		if platform.system().lower() == "windows":
			clarisse_path = os.path.join(os.getenv('APPDATA'), "Isotropix\\Clarisse")
		elif platform.system().lower().startswith("linux"):
			clarisse_path = os.path.join(os.path.expanduser("~"), ".isotropix/clarisse")
		elif platform.system().lower() == "darwin":
			homedir = os.path.expanduser('~')
			clarisse_path = homedir + '/Library/Preferences/Isotropix/Clarisse'

		clarisse_versions = os.listdir(clarisse_path)
		for installed_clarisse in clarisse_versions:			
			if installed_clarisse in supported_clarisse:
				clarisse_dir = os.path.join(clarisse_path, installed_clarisse)
				config_path = os.path.join(clarisse_dir, clarisse_config)
				if not os.path.isfile(config_path):
					return			
				
				file = fileinput.input(config_path, inplace=True)
				for line in file:
					if remove_begin_string in line:
						for _ in range(6): # skip this line and next 4 lines
							next(file, None)
					else:
						print line,

	except Exception as e:
		logger.error("Error removing Quixel Megascans :  %s" %str(e))



def config_livelink():
	
	try :
		is_slot_string = False
		clarisse_path = ""
		clarisse_config = "shelf.cfg"
		if platform.system().lower() == "windows":
			clarisse_path = os.path.join(os.getenv('APPDATA'), "Isotropix\\Clarisse")
		elif platform.system().lower().startswith("linux"):
			clarisse_path = os.path.join(os.path.expanduser("~"), ".isotropix/clarisse")
		elif platform.system().lower() == "darwin":
			homedir = os.path.expanduser('~')
			clarisse_path = homedir + '/Library/Preferences/Isotropix/Clarisse'	

		clarisse_versions = os.listdir(clarisse_path)
		for installed_clarisse in clarisse_versions:
			if installed_clarisse in supported_clarisse:
				clarisse_dir = os.path.join(clarisse_path, installed_clarisse)
				config_path = os.path.join(clarisse_dir, clarisse_config)
				if not os.path.isfile(config_path):
					logger.error("Clarisse configuration file doesnt exist") 
					sys.exit(1)			

				# copy(config_path, base_path)
				config_content = open(config_path, 'r').readlines()
				rewrite_config = open(config_path,'w')
				for line in config_content:
					rewrite_config.write(line)
					if 'slot 0 {' in line:					
						rewrite_config.write(livelink_clarisse_cfg + "\n")
						is_slot_string = True					
				
				rewrite_config.close()			

				if is_slot_string == False:
					additional_string = "slot 0 {\n"
					config_content = open(config_path, 'r').readlines()
					rewrite_config = open(config_path,'w')
					for line in config_content:
						rewrite_config.write(line)
						if 'view_mode 0' in line :
							livelink_clarisse_new = additional_string + livelink_clarisse_cfg + '}\n '
							rewrite_config.write(livelink_clarisse_new + "\n")											

					rewrite_config.close()					
				logger.info('Livelink added to clarisse')

	except Exception as e:
		logger.error("Error writing to Clarisse config") 
		sys.exit(1)

uninstall_livelink()
set_livelink_path()
config_livelink()
